#pragma once

#include <cstdio>
#include <cstdint>

namespace Feather::Network
{

    // Class to read packet data, such as is produced by NetworkMessage
    class PacketReader
    {
    public:
        PacketReader(uint8_t* dataPtr)
          : m_data(dataPtr), m_length(ReadVarInt())
        {
        }

        inline uint8_t ReadByte()
        {
            return m_data[m_offset++];
        }

        inline int ReadVarInt()
        {
            int numRead = 0;
            int result = 0;
            uint8_t read;
            do
            {
                read = ReadByte();
                int value = (read & 0b01111111);
                result |= (value << (7 * numRead));

                numRead++;
                if (numRead > 5)
                {
                    // complain
                    //throw new RuntimeException("VarInt is too big");
                }
            } while ((read & 0b10000000) != 0);

            return result;
        }

        inline uint32_t Length() const { return m_length; }
    private:
        const uint8_t const* m_data;
        const uint32_t       m_length;
        uint32_t             m_offset = 0;
    };

}